﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Windows.Controls;
using Microsoft.Xrm.Sdk.Client;
using Microsoft.Xrm.Sdk;
using Microsoft.Xrm.Sdk.Query;
using System.Windows.Threading;
using System.ComponentModel;

namespace VisualRibbonEditor
{
	/// <summary>
	/// WebResource manager
	/// </summary>
	public static class WebResourcceManager
{
		//dictionary used for caching webresource contents 
		private static Dictionary<string, byte[]> _cache = new Dictionary<string, byte[]>();

		//cached list of names of all webresources
		static string[] _webResouceNames = null;
		

		public static void GetWebResourceContent(string name, Action<byte[]> callback)
		{
			name = name.ToLowerInvariant();
			if (_cache.ContainsKey(name))
			{
				//found the data in the local cache.
				callback(_cache[name]);
			}
			else
			{
				//retrieve the data (in background thread) and then cache it.
				RetrieveWebResource(name, (bits) =>
				{
					_cache[name] = bits;
					callback(bits);
				});
			}
		}

		/// <summary>
		/// Retrieves the contents of a webresource in a background thread.
		/// </summary>
		private static void RetrieveWebResource(string name, Action<byte[]> callback)
		{
			Dispatcher dispatcher = Dispatcher.CurrentDispatcher;
			BackgroundWorker worker = new BackgroundWorker();
			worker.DoWork += (sender, e) =>
			{
				//retrieve the webresource from CRM in a background thread
				var svc = Helpers.GetOrgService();
				QueryExpression q = new QueryExpression
				{
					EntityName = "webresource",
					ColumnSet = new ColumnSet { AllColumns = true }
				};
				q.Criteria = new FilterExpression();
				ConditionExpression cond = new ConditionExpression("name", ConditionOperator.Equal, name);
				q.Criteria.Conditions.Add(cond);

				svc.RetrieveMultipleAsync(q, (ec) =>
				{
					Byte[] bits = null;
					if (ec.Entities.Count > 0)
					{
						Entity webResouce = ec.Entities[0];
						string content = webResouce["content"].ToString();
						bits = Convert.FromBase64String(content);
					}

					dispatcher.Invoke(callback, bits);
				});
			};

			worker.RunWorkerAsync();
		}


		public static void GetAllWebResourceNames(Action<string[]> callback)
		{
			if (_webResouceNames != null)
			{
				callback(_webResouceNames);
			}
			else
			{
				//Retrieve the names from CRM in a background thread
				Dispatcher dispatcher = Dispatcher.CurrentDispatcher;
				BackgroundWorker worker = new BackgroundWorker();
				worker.DoWork += (sender, e) =>
				{
					var svc = Helpers.GetOrgService();

					QueryExpression q = new QueryExpression();
					q.EntityName = "webresource";
					q.ColumnSet = new ColumnSet();
					q.ColumnSet.AddColumn("name");

					svc.RetrieveMultipleAsync(q, (ec) =>
					{
						List<string> names = new List<string>();
						foreach (var ent in ec.Entities)
						{
							names.Add("$webresource:" + ent["name"].ToString());
						}
						
						//cache the names
						_webResouceNames = names.ToArray();

						object[] args = new object[] { _webResouceNames };
						dispatcher.Invoke(callback, args);
					});
				};
				worker.RunWorkerAsync();
			}
		}


		/// <summary>
		/// Sets up an autoCompleteBox to use a list of all image (.png) webresources on the current CRM org. 
		/// </summary>
		/// <param name="boxes">list of autocompleteboxes to set up.</param>
		public static void SetupImageAutoComplete(AutoCompleteBox[] boxes)
		{
			WebResourcceManager.GetAllWebResourceNames(names =>
			{
				foreach (var box in boxes)
				{
					box.FilterMode = AutoCompleteFilterMode.Custom;
					box.ItemsSource = names.Where(x => x.EndsWith(".png", StringComparison.InvariantCultureIgnoreCase));
					box.ItemFilter = new AutoCompleteFilterPredicate<object>(FilterByName);
				}
			});
		}

		public static void SetupScriptAutoComplete(AutoCompleteBox box)
		{
			box.FilterMode = AutoCompleteFilterMode.Custom;
			WebResourcceManager.GetAllWebResourceNames(names =>
			{
				box.ItemsSource = names.Where(x => x.EndsWith(".js", StringComparison.InvariantCultureIgnoreCase));
				box.ItemFilter = new AutoCompleteFilterPredicate<object>(FilterByName);
			});
		}		

		 

		private static bool FilterByName(string s, object o)
		{
			string item = o.ToString().Replace("$webresource:", "").ToLowerInvariant();
			s = s.ToLowerInvariant().Replace("$webresource:", "");

			if (item.StartsWith(s))
			{
				return true;
			}

			int lastSlash = item.LastIndexOf('/');
			if (lastSlash > 0)
			{
				item = item.Substring(lastSlash + 1);
				if (item.StartsWith(s))
				{
					return true;
				}
			}

			return false;
		}
	}
}
